from cuckoo.migration import Migration

from peewee import ProgrammingError
from playhouse.reflection import Introspector
from playhouse.migrate import PostgresqlMigrator

import importlib.util
import os


class Migrator(object):

    def __init__(self, db, path):
        self.db = db
        self._path = path

        self._introspector = Introspector.from_database(self.db)
        self.models = self._introspector.generate_models()
        self.migration_ids = [filename.split('.')[0] for filename in filter(lambda f: f.endswith('.py') and '__init__' not in f, sorted(os.listdir(self._path)))]
    
    def _load_migration(self, m_id):
        module_name = f'{os.path.relpath(self._path).replace("/", ".")}.{m_id}'

        spec = importlib.util.spec_from_file_location(module_name, os.path.join(self._path, f'{m_id}.py'))
        m_mod = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(m_mod)

        return m_mod
    
    def _apply_single_migration(self, pg_migrator, direction, migration_id, fn):
        with self.db.atomic():
            try:
                print(f'Processing migration [{migration_id}].')
                fn(self.db, pg_migrator)

                if direction == 'up':
                    Migration(migration_id=migration_id).save()
                elif direction == 'down':
                    Migration.delete().where(Migration.migration_id == migration_id).execute()
            
            except Exception as e:
                print(f'An error occured while processing migration [{migration_id}]: \n\t- {e}')
                raise RuntimeError
    
    def try_prepare_migrations(self):
        self.db.create_tables([Migration], safe=True)
    
    def apply(self, direction):
        applied_migration_ids = set([m.migration_id for m in Migration.select().execute()])
        pg_migrator = PostgresqlMigrator(self.db)

        if direction == 'up':
            ids = filter(lambda m: m not in applied_migration_ids, self.migration_ids)
        elif direction == 'down':
            ids = filter(lambda m: m in applied_migration_ids, self.migration_ids[::-1])
        
        applied_atleast_one = False
        for m_id in ids:
            applied_atleast_one = True
            fn = getattr(self._load_migration(m_id), direction)

            try:
                self._apply_single_migration(pg_migrator, direction, m_id, fn)
            
            except RuntimeError:
                break
        
        if not applied_atleast_one:
            print('Nothing to do.')

    def run(self, direction):
        print(f'Migrating [{direction}].')
        self.try_prepare_migrations()

        # Apply migrations
        self.apply(direction)
        
        print('Migration complete.')


from cuckoo.cuckoo import Migrator
from cuckoo.db import Database
from cuckoo.migration import MIGRATION_TEMPLATE, Migration

import fire
import os
import time


DEFAULT_MIGRATION_PATH = "./migrations"
DEFAULT_DB_CONFIG = "config.json"


class Runner(object):

    @staticmethod
    def migrate(direction, migration_path=DEFAULT_MIGRATION_PATH, config=DEFAULT_DB_CONFIG):
        db = Database.get_instance(config)
        Migration._meta.database = db
        migrator = Migrator(db, migration_path)
        migrator.run(direction)

    @staticmethod
    def new(name, migration_path=DEFAULT_MIGRATION_PATH):
        timestamp = int(time.time())
        filename = os.path.join(migration_path, str(timestamp) + '_' + str(name) + '.py')

        with open(filename, 'a') as outfile:
            outfile.write(MIGRATION_TEMPLATE)
        
        print(f'Migration [{filename}] created successfully!')


def main():
    fire.Fire(Runner)

if __name__ == '__main__':
    main()


from setuptools import setup

import re


version = ''
with open('cuckoo/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Version is not set')


readme = 'See https://github.com/opfront/cuckoo for details.'

with open('requirements.txt', 'r') as infile:
    requirements = infile.readlines() 


setup(
    name='cuckoo-db',
    version=version,
    packages=['cuckoo'],
    license='MIT',
    author='opfront',
    author_email='dalloriam@gmail.com',
    url='https://github.com/opfront/cuckoo',
    description='Simple migration tool',
    long_description=readme,
    entry_points={
        'console_scripts': [
            'cuckoo=cuckoo.__main__:main'
        ]
    },
    install_requires=requirements
)


from playhouse.postgres_ext import PostgresqlExtDatabase

import json
import os


class Database(object):
    __DB__ = None
    
    @classmethod
    def get_instance(cls, cfg_path):
        if Database.__DB__ is None:
            Database.set_db(cfg_path)
        
        return Database.__DB__
        
    @classmethod
    def set_db(cls, cfg_path):
        with open(cfg_path, 'r') as infile:
            config = json.load(infile)

        Database.__DB__ = PostgresqlExtDatabase(
            database=config['db_name'],
            user=config['user'],
            password=config['password'],
            host=config['host'],
            port=config['port'],
            register_hstore=False
        )